
local BList = {};

BloodEffects = {};

function BloodEffects.createEffect(position, sprayDirection)
    local fx = {};

    fx._splat = lj2d.graphics.createSprite(BloodEffects._splatTex);
    fx._splat:setPosition(position);
    fx._splat:setSize(luajogo.vec2(80, 80));
    fx._splat:setPivot(luajogo.vec2(0.5, 0.5));
    fx._splat:setTextureRegion(BloodEffects._splatRegion);

    local sprite = lj2d.graphics.createSprite(BloodEffects._sprayTex);
    sprite:setPivot(luajogo.vec2(0.5, 0));
    sprite:setPosition(position);
    sprite:setSize(luajogo.vec2(80, 80));
    sprite:setRotation(math.atan2(sprayDirection.y, sprayDirection.x) * luajogo.RAD2DEG - 90);
    fx._anim = SpriteAnimation.create(sprite, BloodEffects._sprayAnim);
    fx._anim:play();

    table.insert(BList, fx);
end

function BloodEffects.load()
    BloodEffects._sprayTex = luajogo.texture.load(luajogo.assets.load("/bloodsplat2_strip13.png"));
    BloodEffects._splatTex = luajogo.texture.load(luajogo.assets.load("/bloodsplat3_strip15.png"));

    BloodEffects._sprayAnim = {
        { time = 0.05, region = { position = luajogo.vec2(480 * 0, 0), size = luajogo.vec2(480, 480) } },
        { time = 0.05, region = { position = luajogo.vec2(480 * 1, 0), size = luajogo.vec2(480, 480) } },
        { time = 0.05, region = { position = luajogo.vec2(480 * 2, 0), size = luajogo.vec2(480, 480) } },
        { time = 0.05, region = { position = luajogo.vec2(480 * 3, 0), size = luajogo.vec2(480, 480) } },
        { time = 0.1, region = { position = luajogo.vec2(480 * 4, 0), size = luajogo.vec2(480, 480) } },
        { time = 0.1, region = { position = luajogo.vec2(480 * 5, 0), size = luajogo.vec2(480, 480) } },
        { time = 0.1, region = { position = luajogo.vec2(480 * 6, 0), size = luajogo.vec2(480, 480) } },
        { time = 0.1, region = { position = luajogo.vec2(480 * 7, 0), size = luajogo.vec2(480, 480) } },
    };
    BloodEffects._splatRegion = { position = luajogo.vec2(1916, 0), size = luajogo.vec2(480, 480) };
end

function BloodEffects.update(deltaTime)
    for _, fx in pairs(BList) do
        if fx._anim then
            fx._anim:update(deltaTime);
            if not fx._anim:isPlaying() then
                fx._anim:destroy();
                fx._anim = nil;
            end
        end
    end
end

function BloodEffects.draw()
    for _, fx in pairs(BList) do
        fx._splat:draw();
        if fx._anim then
            fx._anim:sprite():draw();
        end
    end
end

function BloodEffects.quit()
    for _, fx in pairs(BList) do
        if fx._anim then
            fx._anim:destroy();
        end
        fx._splat:destroy();
    end
    BList = {};
end