
local VERTEX_CODE = [[
#version 330

in vec2 position;
in vec2 uvCoordinates;

out vec2 UV;

uniform mat4 LocalToWorldMatrix;
uniform vec2 ScreenSize;

void main()
{
    gl_Position = LocalToWorldMatrix * vec4(position.xy, 0, 1);
    gl_Position.x = (gl_Position.x / ScreenSize.x) * 2 - 1;
    gl_Position.y = (gl_Position.y / ScreenSize.y) * 2 - 1;
    UV = uvCoordinates;
}
]];
local FRAGMENT_CODE = [[
#version 330

in vec2 UV;

out vec4 outColor;

uniform sampler2D Texture1;
uniform vec4 BaseColor;

void main()
{
    vec4 sampled = vec4(1.0, 1.0, 1.0, texture2D(Texture1, UV).r);
    outColor = BaseColor * sampled;
}
]];

UI = {};

function UI.addScore()
    UI._score = UI._score + 1;
    local scoreVA = UI._font:makeVertexArray(tostring(UI._score));
    luajogo.graphics.updateVertexArray(UI._scoreVA, scoreVA.vbuf, scoreVA.ebuf);
end

function UI.load()
    UI._fontVertexShader = luajogo.graphics.createShader(VERTEX_CODE, luajogo.graphics.VERTEX_SHADER);
    UI._fontFragmentShader = luajogo.graphics.createShader(FRAGMENT_CODE, luajogo.graphics.FRAGMENT_SHADER);
    UI._fontProgram = luajogo.graphics.createShaderProgram(UI._fontVertexShader, UI._fontFragmentShader);

    local fontAsset = luajogo.assets.load("/lcdsolid.ttf");
    UI._font = luajogo.font.load(fontAsset);

    UI._tex = luajogo.graphics.createTexture(UI._font:texture());

    local scoreLabelVA = UI._font:makeVertexArray("Score:");
    UI._scoreLabelVA = luajogo.graphics.createVertexArray(scoreLabelVA.vbuf, scoreLabelVA.ebuf);
    luajogo.graphics.setVertexArrayAttribute(UI._scoreLabelVA, 0, 3);
    luajogo.graphics.setVertexArrayAttribute(UI._scoreLabelVA, 1, 2);
    UI._scoreLabelMat = luajogo.mat4x4();
    UI._scoreLabelMat:translate(luajogo.vec3(10, 10, 0));
    UI._scoreLabelMat:scale(luajogo.vec3(1, -1, 1));

    local scoreVA = UI._font:makeVertexArray("0");
    UI._scoreVA = luajogo.graphics.createVertexArray(scoreVA.vbuf, scoreVA.ebuf);
    luajogo.graphics.setVertexArrayAttribute(UI._scoreVA, 0, 3);
    luajogo.graphics.setVertexArrayAttribute(UI._scoreVA, 1, 2);
    UI._scoreMat = luajogo.mat4x4();
    UI._scoreMat:translate(luajogo.vec3(130, 10, 0));
    UI._scoreMat:scale(luajogo.vec3(1, -1, 1));

    local uDiedVA = UI._font:makeVertexArray("YOU DIED!");
    UI._uDiedVA = luajogo.graphics.createVertexArray(uDiedVA.vbuf, uDiedVA.ebuf);
    luajogo.graphics.setVertexArrayAttribute(UI._uDiedVA, 0, 3);
    luajogo.graphics.setVertexArrayAttribute(UI._uDiedVA, 1, 2);
    UI._uDiedMat = luajogo.mat4x4();
    UI._uDiedMat:translate(luajogo.vec3(lj2d.window.size().x / 2 - 100, lj2d.window.size().y / 2, 0));
    UI._uDiedMat:scale(luajogo.vec3(1, -1, 1));

    local restartVA = UI._font:makeVertexArray("Press R to restart");
    UI._restartVA = luajogo.graphics.createVertexArray(restartVA.vbuf, restartVA.ebuf);
    luajogo.graphics.setVertexArrayAttribute(UI._restartVA, 0, 3);
    luajogo.graphics.setVertexArrayAttribute(UI._restartVA, 1, 2);
    UI._restartMat = luajogo.mat4x4();
    UI._restartMat:translate(luajogo.vec3(lj2d.window.size().x / 2 - 200, lj2d.window.size().y / 2 - 60, 0));
    UI._restartMat:scale(luajogo.vec3(1, -1, 1));

    UI._score = 0;
end

function UI.draw()
    luajogo.graphics.useShaderProgram(UI._fontProgram);
    luajogo.graphics.setShaderParam(UI._fontProgram, "BaseColor", luajogo.color("white"));
    luajogo.graphics.setShaderParam(UI._fontProgram, "ScreenSize", lj2d.window.size());
    luajogo.graphics.useTexture(UI._tex);

    luajogo.graphics.useVertexArray(UI._scoreLabelVA);
    luajogo.graphics.setShaderParam(UI._fontProgram, "LocalToWorldMatrix", UI._scoreLabelMat);
    luajogo.graphics.draw();

    luajogo.graphics.useVertexArray(UI._scoreVA);
    luajogo.graphics.setShaderParam(UI._fontProgram, "LocalToWorldMatrix", UI._scoreMat);
    luajogo.graphics.draw();

    if PLAYER:isDead() then
        luajogo.graphics.useVertexArray(UI._uDiedVA);
        luajogo.graphics.setShaderParam(UI._fontProgram, "LocalToWorldMatrix", UI._uDiedMat);
        luajogo.graphics.draw();
        luajogo.graphics.useVertexArray(UI._restartVA);
        luajogo.graphics.setShaderParam(UI._fontProgram, "LocalToWorldMatrix", UI._restartMat);
        luajogo.graphics.draw();
    end
end

function UI.quit()
    UI._font = nil;

    luajogo.graphics.deleteVertexArray(UI._scoreLabelVA);
    luajogo.graphics.deleteVertexArray(UI._scoreVA);
    luajogo.graphics.deleteVertexArray(UI._uDiedVA);
    luajogo.graphics.deleteVertexArray(UI._restartVA);

    luajogo.graphics.deleteTexture(UI._tex);

    luajogo.graphics.deleteShaderProgram(UI._fontProgram);
    luajogo.graphics.deleteShader(UI._fontVertexShader);
    luajogo.graphics.deleteShader(UI._fontFragmentShader);
end