
local VERTEX_CODE = [[
#version 330

in vec2 position;
in vec2 uvCoordinates;

out vec2 UV;

uniform float GlobalTime;
uniform float TiltAngle;

mat4 rotationMatrix(vec3 axis, float angle)
{
    axis = normalize(axis);
    float s = sin(angle);
    float c = cos(angle);
    float oc = 1.0 - c;
    
    return mat4(oc * axis.x * axis.x + c,           oc * axis.x * axis.y - axis.z * s,  oc * axis.z * axis.x + axis.y * s,  0.0,
                oc * axis.x * axis.y + axis.z * s,  oc * axis.y * axis.y + c,           oc * axis.y * axis.z - axis.x * s,  0.0,
                oc * axis.z * axis.x - axis.y * s,  oc * axis.y * axis.z + axis.x * s,  oc * axis.z * axis.z + c,           0.0,
                0.0,                                0.0,                                0.0,                                1.0);
}

void main()
{
    float angle = (sin(GlobalTime) * 2 - 1) * TiltAngle;
    vec4 pos = rotationMatrix(vec3(0, 0, 1), angle) * vec4(position.xy, 0, 1);

    gl_Position = vec4(pos.xy, 0, 1);
    UV = uvCoordinates;
}
]];

local FRAGMENT_CODE = [[
#version 330

in vec2 UV;

out vec4 outColor;

uniform sampler2D Texture1;
uniform float GlobalTime;
uniform float Strength;

highp float rand(vec2 co)
{
    highp float a = 12.9898;
    highp float b = 78.233;
    highp float c = 43758.5453;
    highp float dt= dot(co.xy ,vec2(a,b));
    highp float sn= mod(dt,3.14);
    return fract(sin(sn) * c);
}

void main()
{
    vec2 uv = UV;
    // Flip Y Axis
    //uv.y = -uv.y;
    
    highp float magnitude = 0.0009;
    
    // Set up offset
    vec2 offsetRedUV = uv;
    offsetRedUV.x = uv.x + rand(vec2(GlobalTime*0.03,uv.y*0.42)) * 0.001;
    offsetRedUV.x += sin(rand(vec2(GlobalTime*0.2, uv.y)))*magnitude;
    
    vec2 offsetGreenUV = uv;
    offsetGreenUV.x = uv.x + rand(vec2(GlobalTime*0.004,uv.y*0.002)) * 0.004;
    offsetGreenUV.x += sin(GlobalTime*9.0)*magnitude;
    
    vec2 offsetBlueUV = uv;
    offsetBlueUV.x = uv.y;
    offsetBlueUV.x += rand(vec2(cos(GlobalTime*0.01),sin(uv.y)));
    
    // Load Texture
    float r = texture2D(Texture1, offsetRedUV).r;
    float g = texture2D(Texture1, offsetGreenUV).g;
    float b = texture2D(Texture1, uv).b;

    vec4 normalColor = texture2D(Texture1, uv);
    vec4 changedColor = vec4(r, g, b, 1);
    
    outColor = normalColor + (changedColor - normalColor) * Strength;
}
]];

--local FRAGMENT_CODE = [[
--#version 330
--
--in vec2 UV;
--
--out vec4 outColor;
--
--uniform sampler2D Texture1;
--
--void main()
--{
--    outColor = texture2D(Texture1, UV) * vec4(1, 0, 0, 1);
--}
--]];

VHSEffect = {};

function VHSEffect.buff()
    VHSEffect._strength1 = VHSEffect._strength1 + VHSEffect._strength1gain;
    if VHSEffect._strength1 > VHSEffect._strength1max then
        VHSEffect._strength1 = VHSEffect._strength1max;
    end
    VHSEffect._strength2 = VHSEffect._strength2 + VHSEffect._strength2gain;
    if VHSEffect._strength2 > VHSEffect._strength2max then
        VHSEffect._strength2 = VHSEffect._strength2max;
    end
    VHSEffect._tilt = VHSEffect._tilt + VHSEffect._tiltGain;
    if VHSEffect._tilt > VHSEffect._tiltMax then
        VHSEffect._tilt = VHSEffect._tiltMax;
    end
end

function VHSEffect.load()
    local size = lj2d.window.size();

    VHSEffect._vertex = luajogo.graphics.createShader(VERTEX_CODE, luajogo.graphics.VERTEX_SHADER);
    VHSEffect._fragment = luajogo.graphics.createShader(FRAGMENT_CODE, luajogo.graphics.FRAGMENT_SHADER);
    VHSEffect._pinkFragment = luajogo.graphics.createShader([[
    #version 330
    
    in vec2 UV;
    
    out vec4 outColor;
    
    uniform sampler2D Texture1;
    uniform float Strength;
    
    void main()
    {
        outColor = vec4(1, 0.441, 0.705, Strength);
    }
    ]], luajogo.graphics.FRAGMENT_SHADER);

    VHSEffect._program = luajogo.graphics.createShaderProgram(VHSEffect._vertex, VHSEffect._fragment);
    VHSEffect._pinkProgram = luajogo.graphics.createShaderProgram(VHSEffect._vertex, VHSEffect._pinkFragment);

    VHSEffect._tex = luajogo.graphics.createTexture(size.x, size.y, 4);

    VHSEffect._va = luajogo.graphics.createVertexArray({
        -1, -1, 0, 0,
        1, -1, 1, 0,
        -1, 1, 0, 1,
        1, 1, 1, 1,
    }, {
        0, 1, 3,
        0, 2, 3
    });
    luajogo.graphics.setVertexArrayAttribute(VHSEffect._va, 0, 2);
    luajogo.graphics.setVertexArrayAttribute(VHSEffect._va, 1, 2);

    VHSEffect._fbuf = luajogo.graphics.createFrameBuffer(VHSEffect._tex, size.x, size.y);

    VHSEffect._strength1 = 0.2;
    VHSEffect._strength1gain = 0.05;
    VHSEffect._strength1max = 1;

    VHSEffect._strength2 = 0.05;
    VHSEffect._strength2gain = 0.01;
    VHSEffect._strength2max = 0.4;

    VHSEffect._tilt = 0;
    VHSEffect._tiltGain = 0.1;
    VHSEffect._tiltMax = 30;
end

function VHSEffect.captureFrameBuffer()
    luajogo.graphics.useFrameBuffer(VHSEffect._fbuf);
end

function VHSEffect.draw()
    luajogo.graphics.useFrameBuffer(0);

    luajogo.graphics.useTexture(VHSEffect._tex);
    luajogo.graphics.useVertexArray(VHSEffect._va);

    luajogo.graphics.useShaderProgram(VHSEffect._program);
    luajogo.graphics.setShaderParam(VHSEffect._program, "GlobalTime", lj2d.window.time());
    luajogo.graphics.setShaderParam(VHSEffect._program, "TiltAngle", VHSEffect._tilt * luajogo.DEG2RAD);
    luajogo.graphics.setShaderParam(VHSEffect._program, "Strength", VHSEffect._strength1);
    luajogo.graphics.draw();

    luajogo.graphics.useShaderProgram(VHSEffect._pinkProgram);
    luajogo.graphics.setShaderParam(VHSEffect._pinkProgram, "Strength", VHSEffect._strength2);
    luajogo.graphics.draw();
end

function VHSEffect.quit()
    luajogo.graphics.deleteShader(VHSEffect._vertex);
    luajogo.graphics.deleteShader(VHSEffect._fragment);
    luajogo.graphics.deleteShaderProgram(VHSEffect._program);

    luajogo.graphics.deleteTexture(VHSEffect._tex);

    luajogo.graphics.deleteVertexArray(VHSEffect._va);

    luajogo.graphics.deleteFrameBuffer(VHSEffect._fbuf);
end